@@ -1,5 +1,5 @@
 //
-// $Id: indexer.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: indexer.cpp 3461 2012-10-19 09:48:07Z kevg $
 //
 
 //
@@ -43,7 +43,7 @@ bool			g_bRotate		= false;
 bool			g_bRotateEach	= false;
 bool			g_bBuildFreqs	= false;
 
-int				g_iMemLimit				= 0;
+int				g_iMemLimit				= 32*1024*1024;
 int				g_iMaxXmlpipe2Field		= 0;
 int				g_iWriteBuffer			= 0;
 int				g_iMaxFileFieldBuffer	= 1024*1024;
@@ -336,7 +336,7 @@ bool ParseMultiAttr ( const char * sBuf, CSphColumnInfo & tAttr, const char * sS
 	LOC_SPACE0(); LOC_TOK();
 	if ( LOC_TOKEQ("uint") )				tAttr.m_eAttrType = SPH_ATTR_UINT32SET;
 	else if ( LOC_TOKEQ("timestamp") )		tAttr.m_eAttrType = SPH_ATTR_UINT32SET;
-	else if ( LOC_TOKEQ("bigint") )			tAttr.m_eAttrType = SPH_ATTR_UINT64SET;
+	else if ( LOC_TOKEQ("bigint") )			tAttr.m_eAttrType = SPH_ATTR_INT64SET;
 	else									LOC_ERR ( "attr type ('uint' or 'timestamp' or 'bigint')", sTok );
 
 	// handle ATTR-NAME
@@ -397,6 +397,11 @@ bool ParseMultiAttr ( const char * sBuf, CSphColumnInfo & tAttr, const char * sS
 	if ( hSource.Exists(_key) && hSource[_key].intval() ) \
 		_arg = hSource[_key].intval();
 
+// get int64_t
+#define LOC_GETL(_arg,_key) \
+	if ( hSource.Exists(_key) ) \
+		_arg = hSource[_key].int64val();
+
 // get bool
 #define LOC_GETB(_arg,_key) \
 	if ( hSource.Exists(_key) ) \
@@ -586,7 +591,7 @@ bool SqlParamsConfigure ( CSphSourceParams_SQL & tParams, const CSphConfigSectio
 	LOC_GETA ( tParams.m_dQueryPost,		"sql_query_post" );
 	LOC_GETS ( tParams.m_sQueryRange,		"sql_query_range" );
 	LOC_GETA ( tParams.m_dQueryPostIndex,	"sql_query_post_index" );
-	LOC_GETI ( tParams.m_iRangeStep,		"sql_range_step" );
+	LOC_GETL ( tParams.m_iRangeStep,		"sql_range_step" );
 	LOC_GETS ( tParams.m_sQueryKilllist,	"sql_query_killlist" );
 
 	LOC_GETI ( tParams.m_iRangedThrottle,	"sql_ranged_throttle" );
@@ -837,6 +842,7 @@ CSphSource * SpawnSource ( const CSphConfigSection & hSource, const char * sSour
 #undef LOC_CHECK
 #undef LOC_GETS
 #undef LOC_GETI
+#undef LOC_GETL
 #undef LOC_GETA
 
 //////////////////////////////////////////////////////////////////////////
@@ -1005,7 +1011,6 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName, const
 	// parse all sources
 	CSphVector<CSphSource*> dSources;
 	bool bGotAttrs = false;
-	bool bGotJoinedFields = false;
 	bool bSpawnFailed = false;
 
 	for ( CSphVariant * pSourceName = hIndex("source"); pSourceName; pSourceName = pSourceName->m_pNext )
@@ -1027,9 +1032,6 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName, const
 		if ( pSource->HasAttrsConfigured() )
 			bGotAttrs = true;
 
-		if ( pSource->HasJoinedFields() )
-			bGotJoinedFields = true;
-
 		// strip_html, index_html_attrs
 		CSphString sError;
 		if ( bStripOverride )
@@ -1155,12 +1157,6 @@ bool DoIndex ( const CSphConfigSection & hIndex, const char * sIndexName, const
 			exit ( 1 );
 		}
 
-		if ( bGotJoinedFields && tSettings.m_eDocinfo==SPH_DOCINFO_INLINE )
-		{
-			fprintf ( stdout, "FATAL: index '%s': got joined fields, but docinfo is 'inline' (fix your config file).\n", sIndexName );
-			exit ( 1 );
-		}
-
 		pIndex->SetProgressCallback ( ShowProgress );
 		if ( bInplaceEnable )
 			pIndex->SetInplaceSettings ( iHitGap, iDocinfoGap, fRelocFactor, fWriteFactor );
@@ -1436,14 +1432,40 @@ void SetSignalHandlers ()
 
 #endif // USE_WINDOWS
 
-bool SendRotate ( int iPID, bool bForce )
+bool SendRotate ( const CSphConfig & hConf, bool bForce )
 {
-	if ( iPID<0 )
-		return false;
-
 	if ( !( g_bRotate && ( g_bRotateEach || bForce ) ) )
 		return false;
 
+	int iPID = -1;
+	// load config
+	if ( !hConf.Exists ( "searchd" ) )
+	{
+		fprintf ( stdout, "WARNING: 'searchd' section not found in config file.\n" );
+		return false;
+	}
+
+	const CSphConfigSection & hSearchd = hConf["searchd"]["searchd"];
+	if ( !hSearchd.Exists ( "pid_file" ) )
+	{
+		fprintf ( stdout, "WARNING: 'pid_file' parameter not found in 'searchd' config section.\n" );
+		return false;
+	}
+
+	// read in PID
+	FILE * fp = fopen ( hSearchd["pid_file"].cstr(), "r" );
+	if ( !fp )
+	{
+		fprintf ( stdout, "WARNING: failed to open pid_file '%s'.\n", hSearchd["pid_file"].cstr() );
+		return false;
+	}
+	if ( fscanf ( fp, "%d", &iPID )!=1 || iPID<=0 )
+	{
+		fprintf ( stdout, "WARNING: failed to scanf pid from pid_file '%s'.\n", hSearchd["pid_file"].cstr() );
+		return false;
+	}
+	fclose ( fp );
+
 #if USE_WINDOWS
 	char szPipeName[64];
 	snprintf ( szPipeName, sizeof(szPipeName), "\\\\.\\pipe\\searchd_%d", iPID );
@@ -1476,7 +1498,7 @@ bool SendRotate ( int iPID, bool bForce )
 		BYTE uWrite = 0;
 		BOOL bResult = WriteFile ( hPipe, &uWrite, 1, &uWritten, NULL );
 		if ( bResult )
-			fprintf ( stdout, "rotating indices: succesfully sent SIGHUP to searchd (pid=%d).\n", iPID );
+			fprintf ( stdout, "rotating indices: successfully sent SIGHUP to searchd (pid=%d).\n", iPID );
 		else
 			fprintf ( stdout, "WARNING: failed to send SIGHUP to searchd (pid=%d, GetLastError()=%d)\n", iPID, GetLastError () );
 
@@ -1488,7 +1510,7 @@ bool SendRotate ( int iPID, bool bForce )
 	if ( iErr==0 )
 	{
 		if ( !g_bQuiet )
-			fprintf ( stdout, "rotating indices: succesfully sent SIGHUP to searchd (pid=%d).\n", iPID );
+			fprintf ( stdout, "rotating indices: successfully sent SIGHUP to searchd (pid=%d).\n", iPID );
 	} else
 	{
 		switch ( errno )
@@ -1541,8 +1563,8 @@ int main ( int argc, char ** argv )
 			dMergeDstFilters.Add();
 			dMergeDstFilters.Last().m_eType = SPH_FILTER_RANGE;
 			dMergeDstFilters.Last().m_sAttrName = argv[i+1];
-			dMergeDstFilters.Last().m_uMinValue = (SphAttr_t) strtoull ( argv[i+2], NULL, 10 );
-			dMergeDstFilters.Last().m_uMaxValue = (SphAttr_t) strtoull ( argv[i+3], NULL, 10 );
+			dMergeDstFilters.Last().m_iMinValue = strtoll ( argv[i+2], NULL, 10 );
+			dMergeDstFilters.Last().m_iMaxValue = strtoll ( argv[i+3], NULL, 10 );
 			i += 3;
 
 		} else if ( strcasecmp ( argv[i], "--buildstops" )==0 && (i+2)<argc )
@@ -1586,7 +1608,7 @@ int main ( int argc, char ** argv )
 		{
 			bVerbose = true;
 
-		} else if ( ( argv[i][0]>='a' && argv[i][0]<='z' ) || ( argv[i][0]>='A' && argv[i][0]<='Z' ) )
+		} else if ( isalnum ( argv[i][0] ) || argv[i][0]=='_' )
 		{
 			dIndexes.Add ( argv[i] );
 
@@ -1645,8 +1667,8 @@ int main ( int argc, char ** argv )
 				"\t\t\tfilter 'dst-index' on merge, keep only those documents\n"
 				"\t\t\twhere 'attr' is between 'min' and 'max' (inclusive)\n"
 				"--merge-klists\n"
-				"--merge-killlists\tmerge src and dst kill-lists (default is to\n"
-				"\t\t\tapply src kill-list to dst index)\n"
+				"--merge-killlists\tmerge src and dst k-lists (default is to discard them\n"
+				"\t\t\tafter merge; note that src k-list applies anyway)\n"
 				"--dump-rows <FILE>\tdump indexed rows into FILE\n"
 				"--print-queries\t\tprint SQL queries (for debugging)\n"
 				"\n"
@@ -1677,12 +1699,11 @@ int main ( int argc, char ** argv )
 	if ( !hConf ( "source" ) )
 		sphDie ( "no indexes found in config file '%s'", sOptConfig );
 
-	g_iMemLimit = 0;
 	if ( hConf("indexer") && hConf["indexer"]("indexer") )
 	{
 		CSphConfigSection & hIndexer = hConf["indexer"]["indexer"];
 
-		g_iMemLimit = hIndexer.GetSize ( "mem_limit", 0 );
+		g_iMemLimit = hIndexer.GetSize ( "mem_limit", g_iMemLimit );
 		g_iMaxXmlpipe2Field = hIndexer.GetSize ( "max_xmlpipe2_field", 2*1024*1024 );
 		g_iWriteBuffer = hIndexer.GetSize ( "write_buffer", 1024*1024 );
 		g_iMaxFileFieldBuffer = Max ( 1024*1024, hIndexer.GetSize ( "max_file_field_buffer", 8*1024*1024 ) );
@@ -1703,40 +1724,6 @@ int main ( int argc, char ** argv )
 		sphSetThrottling ( hIndexer.GetInt ( "max_iops", 0 ), hIndexer.GetSize ( "max_iosize", 0 ) );
 	}
 
-	int iPID = -1;
-	while ( g_bRotate )
-	{
-		// load config
-		if ( !hConf.Exists ( "searchd" ) )
-		{
-			fprintf ( stdout, "WARNING: 'searchd' section not found in config file.\n" );
-			break;
-		}
-
-		const CSphConfigSection & hSearchd = hConf["searchd"]["searchd"];
-		if ( !hSearchd.Exists ( "pid_file" ) )
-		{
-			fprintf ( stdout, "WARNING: 'pid_file' parameter not found in 'searchd' config section.\n" );
-			break;
-		}
-
-		// read in PID
-		FILE * fp = fopen ( hSearchd["pid_file"].cstr(), "r" );
-		if ( !fp )
-		{
-			fprintf ( stdout, "WARNING: failed to open pid_file '%s'.\n", hSearchd["pid_file"].cstr() );
-			break;
-		}
-		if ( fscanf ( fp, "%d", &iPID )!=1 || iPID<=0 )
-		{
-			fprintf ( stdout, "WARNING: failed to scanf pid from pid_file '%s'.\n", hSearchd["pid_file"].cstr() );
-			break;
-		}
-		fclose ( fp );
-
-		break;
-	}
-
 	/////////////////////
 	// index each index
 	////////////////////
@@ -1774,7 +1761,7 @@ int main ( int argc, char ** argv )
 		{
 			bool bLastOk = DoIndex ( hConf["index"].IterateGet (), hConf["index"].IterateGetKey().cstr(), hConf["source"], bVerbose, fpDumpRows );
 			bIndexedOk |= bLastOk;
-			if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( iPID, false ) )
+			if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( hConf, false ) )
 				tmRotated = sphMicroTimer();
 		}
 	} else
@@ -1788,7 +1775,7 @@ int main ( int argc, char ** argv )
 			{
 				bool bLastOk = DoIndex ( hConf["index"][dIndexes[i]], dIndexes[i], hConf["source"], bVerbose, fpDumpRows );
 				bIndexedOk |= bLastOk;
-				if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( iPID, false ) )
+				if ( bLastOk && ( sphMicroTimer() - tmRotated > ROTATE_MIN_INTERVAL ) && SendRotate ( hConf, false ) )
 					tmRotated = sphMicroTimer();
 			}
 		}
@@ -1810,7 +1797,7 @@ int main ( int argc, char ** argv )
 
 	if ( bIndexedOk && g_bRotate )
 	{
-		if ( !SendRotate ( iPID, true ) )
+		if ( !SendRotate ( hConf, true ) )
 			fprintf ( stdout, "WARNING: indices NOT rotated.\n" );
 	}
 
@@ -1822,5 +1809,5 @@ int main ( int argc, char ** argv )
 }
 
 //
-// $Id: indexer.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: indexer.cpp 3461 2012-10-19 09:48:07Z kevg $
 //
