@@ -1,5 +1,5 @@
 //
-// $Id: sphinxexcerpt.h 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxexcerpt.h 3205 2012-05-04 17:31:58Z tomat $
 //
 
 //
@@ -18,6 +18,14 @@
 
 #include "sphinx.h"
 
+enum ESphSpz
+{
+	SPH_SPZ_NONE		= 0,
+	SPH_SPZ_SENTENCE	= 1UL<<0,
+	SPH_SPZ_PARAGRAPH	= 1UL<<1,
+	SPH_SPZ_ZONE		= 1UL<<2
+};
+
 /// a query to generate an excerpt
 /// everything string is expected to be UTF-8
 struct ExcerptQuery_t
@@ -34,7 +42,6 @@ public:
 	int				m_iLimitPassages;	///< max passages in snippet
 	int				m_iAround;			///< how much words to highlight around each match
 	int				m_iPassageId;		///< current %PASSAGE_ID% counter value (must start at 1)
-	int				m_iPassageBoundary;	///< passage boundary mode
 	bool			m_bRemoveSpaces;	///< whether to collapse whitespace
 	bool			m_bExactPhrase;		///< whether to highlight exact phrase matches only
 	bool			m_bUseBoundaries;	///< whether to extract passages by phrase boundaries setup in tokenizer
@@ -58,17 +65,22 @@ public:
 	CSphString		m_sBeforeMatchPassage;
 	CSphString		m_sAfterMatchPassage;
 
+	DWORD			m_ePassageSPZ;
+
 public:
 	ExcerptQuery_t ();
 };
 
+struct XQQuery_t;
+
 /// an excerpt generator
 /// returns a newly allocated string in encoding specified by tokenizer on success
 /// returns NULL on failure
-char * sphBuildExcerpt ( ExcerptQuery_t &, CSphDict *, ISphTokenizer *, const CSphSchema *, CSphIndex *, CSphString & sError, const CSphHTMLStripper *, ISphTokenizer * );
+char * sphBuildExcerpt ( ExcerptQuery_t & tOptions, const CSphIndex * pIndex, const CSphHTMLStripper * pStripper, const XQQuery_t & tExtQuery,
+						DWORD eExtQuerySPZ, CSphString & sError, CSphDict * pDict, ISphTokenizer * pDocTokenizer, ISphTokenizer * pQueryTokenizer );
 
 #endif // _sphinxexcerpt_
 
 //
-// $Id: sphinxexcerpt.h 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxexcerpt.h 3205 2012-05-04 17:31:58Z tomat $
 //
