@@ -1,5 +1,5 @@
 //
-// $Id: sphinxfilter.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxfilter.cpp 3435 2012-10-08 08:49:47Z kevg $
 //
 
 //
@@ -104,18 +104,18 @@ bool IFilter_Values::EvalBlockValues ( SphAttr_t uBlockMin, SphAttr_t uBlockMax
 /// range
 struct IFilter_Range: virtual ISphFilter
 {
-	SphAttr_t m_uMinValue;
-	SphAttr_t m_uMaxValue;
+	SphAttr_t m_iMinValue;
+	SphAttr_t m_iMaxValue;
 
 	virtual void SetRange ( SphAttr_t tMin, SphAttr_t tMax )
 	{
-		m_uMinValue = tMin;
-		m_uMaxValue = tMax;
+		m_iMinValue = tMin;
+		m_iMaxValue = tMax;
 	}
 
 	bool EvalRange ( const SphAttr_t uValue ) const
 	{
-		return uValue>=m_uMinValue && uValue<=m_uMaxValue;
+		return uValue>=m_iMinValue && uValue<=m_iMaxValue;
 	}
 };
 
@@ -184,7 +184,7 @@ struct Filter_Range: public IFilter_Attr, IFilter_Range
 
 		SphAttr_t uBlockMin = sphGetRowAttr ( DOCINFO2ATTRS ( pMinDocinfo ), m_tLocator );
 		SphAttr_t uBlockMax = sphGetRowAttr ( DOCINFO2ATTRS ( pMaxDocinfo ), m_tLocator );
-		return (!( m_uMaxValue<uBlockMin || m_uMinValue>uBlockMax )); // not-reject
+		return (!( m_iMaxValue<uBlockMin || m_iMinValue>uBlockMax )); // not-reject
 	}
 };
 
@@ -252,10 +252,16 @@ struct Filter_IdValues: public IFilter_Values
 
 struct Filter_IdRange: public IFilter_Range
 {
+	virtual void SetRange ( SphAttr_t tMin, SphAttr_t tMax )
+	{
+		m_iMinValue = (SphDocID_t)Max ( 0, (SphDocID_t)tMin );
+		m_iMaxValue = tMax;
+	}
+
 	virtual bool Eval ( const CSphMatch & tMatch ) const
 	{
 		const SphDocID_t uID = tMatch.m_iDocID;
-		return uID>=(SphDocID_t)m_uMinValue && uID<=(SphDocID_t)m_uMaxValue;
+		return uID>=(SphDocID_t)m_iMinValue && uID<=(SphDocID_t)m_iMaxValue;
 	}
 
 	virtual bool EvalBlock ( const DWORD * pMinDocinfo, const DWORD * pMaxDocinfo ) const
@@ -263,7 +269,7 @@ struct Filter_IdRange: public IFilter_Range
 		const SphDocID_t uBlockMin = DOCINFO2ID ( pMinDocinfo );
 		const SphDocID_t uBlockMax = DOCINFO2ID ( pMaxDocinfo );
 
-		return (!( (SphDocID_t)m_uMaxValue<uBlockMin || (SphDocID_t)m_uMinValue>uBlockMax ));
+		return (!( (SphDocID_t)m_iMaxValue<uBlockMin || (SphDocID_t)m_iMinValue>uBlockMax ));
 	}
 
 	Filter_IdRange ()
@@ -350,24 +356,24 @@ bool Filter_MVAValues<true>::MvaEval ( const DWORD * pMva, const DWORD * pMvaMax
 	const SphAttr_t * pFilter = m_pValues;
 	const SphAttr_t * pFilterMax = pFilter + m_iValueCount;
 
-	const uint64_t * L = (const uint64_t *)pMva;
-	const uint64_t * R = (const uint64_t *)( pMvaMax - 2 );
+	const int64_t * L = (const int64_t *)pMva;
+	const int64_t * R = (const int64_t *)( pMvaMax - 2 );
 	for ( ; pFilter < pFilterMax; pFilter++ )
 	{
-		uint64_t uFilter = *pFilter;
+		int64_t uFilter = *pFilter;
 		while ( L<=R )
 		{
-			const uint64_t * pVal = L + (R - L) / 2;
-			uint64_t uMva = MVA_UPSIZE ( (const DWORD *)pVal );
+			const int64_t * pVal = L + (R - L) / 2;
+			int64_t iMva = MVA_UPSIZE ( (const DWORD *)pVal );
 
-			if ( uFilter > uMva )
+			if ( uFilter > iMva )
 				L = pVal + 1;
-			else if ( uFilter < uMva )
+			else if ( uFilter < iMva )
 				R = pVal - 1;
 			else
 				return true;
 		}
-		R = (const uint64_t *)( pMvaMax - 2 );
+		R = (const int64_t *)( pMvaMax - 2 );
 	}
 	return false;
 }
@@ -398,42 +404,42 @@ bool Filter_MVARange<false>::MvaEval ( const DWORD * pMva, const DWORD * pMvaMax
 	while ( L<=R )
 	{
 		const DWORD * m = L + (R - L) / 2;
-		if ( m_uMinValue > *m )
+		if ( m_iMinValue > *m )
 			L = m + 1;
-		else if ( m_uMinValue < *m )
+		else if ( m_iMinValue < *m )
 			R = m - 1;
 		else
 			return true;
 	}
 	if ( L==pMvaMax )
 		return false;
-	return *L<=m_uMaxValue;
+	return *L<=m_iMaxValue;
 }
 
 
 template<>
 bool Filter_MVARange<true>::MvaEval ( const DWORD * pMva, const DWORD * pMvaMax ) const
 {
-	const uint64_t * L = (const uint64_t *)pMva;
-	const uint64_t * R = (const uint64_t *)( pMvaMax - 2 );
+	const int64_t * L = (const int64_t *)pMva;
+	const int64_t * R = (const int64_t *)( pMvaMax - 2 );
 
 	while ( L<=R )
 	{
-		const uint64_t * pVal = L + (R - L) / 2;
-		uint64_t uMva = MVA_UPSIZE ( (const DWORD *)pVal );
+		const int64_t * pVal = L + (R - L) / 2;
+		int64_t iMva = MVA_UPSIZE ( (const DWORD *)pVal );
 
-		if ( (uint64_t)m_uMinValue>uMva )
+		if ( m_iMinValue>iMva )
 			L = pVal + 1;
-		else if ( (uint64_t)m_uMinValue < uMva )
+		else if ( m_iMinValue < iMva )
 			R = pVal - 1;
 		else
 			return true;
 	}
-	if ( L==(const uint64_t *)pMvaMax )
+	if ( L==(const int64_t *)pMvaMax )
 		return false;
 
-	uint64_t uMvaL = MVA_UPSIZE ( (const DWORD *)L );
-	return uMvaL<=(uint64_t)m_uMaxValue;
+	int64_t iMvaL = MVA_UPSIZE ( (const DWORD *)L );
+	return iMvaL<=m_iMaxValue;
 }
 
 
@@ -587,18 +593,18 @@ static inline ISphFilter * ReportError ( CSphString & sError, const char * sMess
 static ISphFilter * CreateFilter ( ESphAttr eAttrType, ESphFilter eFilterType, CSphString & sError )
 {
 	// MVA
-	if ( eAttrType==SPH_ATTR_UINT32SET || eAttrType==SPH_ATTR_UINT64SET )
+	if ( eAttrType==SPH_ATTR_UINT32SET || eAttrType==SPH_ATTR_INT64SET )
 	{
 		switch ( eFilterType )
 		{
 		case SPH_FILTER_VALUES:
-			if ( eAttrType==SPH_ATTR_UINT64SET )
+			if ( eAttrType==SPH_ATTR_INT64SET )
 				return new Filter_MVAValues<true>();
 			else
 				return new Filter_MVAValues<false>();
 
 		case SPH_FILTER_RANGE:
-			if ( eAttrType==SPH_ATTR_UINT64SET )
+			if ( eAttrType==SPH_ATTR_INT64SET )
 				return new Filter_MVARange<true>();
 			else
 				return new Filter_MVARange<false>();
@@ -631,7 +637,15 @@ ISphFilter * sphCreateFilter ( const CSphFilterSettings & tSettings, const CSphS
 	// try to create filter on special attribute
 	const CSphString & sAttrName = tSettings.m_sAttrName;
 	if ( sAttrName.Begins("@") )
+	{
+		// HAVING is not implemented yet
+		if ( sAttrName=="@groupby" || sAttrName=="@count" || sAttrName=="@distinct" )
+		{
+			sError.SetSprintf ( "unsupported filter column '%s'", sAttrName.cstr() );
+			return NULL;
+		}
 		pFilter = CreateSpecialFilter ( sAttrName, tSettings.m_eType );
+	}
 
 	// fetch column info
 	const CSphColumnInfo * pAttr = NULL;
@@ -648,6 +662,12 @@ ISphFilter * sphCreateFilter ( const CSphFilterSettings & tSettings, const CSphS
 		assert ( !pFilter );
 
 		pAttr = &tSchema.GetAttr(iAttr);
+		// HAVING is not implemented yet
+		if ( pAttr->m_eAggrFunc!=SPH_AGGR_NONE )
+		{
+			sError.SetSprintf ( "unsupported filter '%s' on aggregate column", sAttrName.cstr() );
+			return NULL;
+		}
 		pFilter = CreateFilter ( pAttr->m_eAttrType, tSettings.m_eType, sError );
 	}
 
@@ -657,7 +677,7 @@ ISphFilter * sphCreateFilter ( const CSphFilterSettings & tSettings, const CSphS
 		if ( pAttr )
 			pFilter->SetLocator ( pAttr->m_tLocator );
 
-		pFilter->SetRange ( tSettings.m_uMinValue, tSettings.m_uMaxValue );
+		pFilter->SetRange ( tSettings.m_iMinValue, tSettings.m_iMaxValue );
 		pFilter->SetRangeFloat ( tSettings.m_fMinValue, tSettings.m_fMaxValue );
 		pFilter->SetMVAStorage ( pMvaPool );
 
@@ -691,5 +711,5 @@ ISphFilter * sphJoinFilters ( ISphFilter * pA, ISphFilter * pB )
 }
 
 //
-// $Id: sphinxfilter.cpp 3087 2012-01-30 23:07:35Z shodan $
+// $Id: sphinxfilter.cpp 3435 2012-10-08 08:49:47Z kevg $
 //
