@@ -1,5 +1,5 @@
 //
-// $Id: sphinxint.h 3117 2012-02-22 20:30:36Z tomat $
+// $Id: sphinxint.h 3351 2012-08-19 13:48:59Z klirichek $
 //
 
 //
@@ -78,8 +78,8 @@ public:
 
 	void			SetBufferSize ( int iBufferSize );	///< tune write cache size; must be called before OpenFile() or SetFile()
 
-	bool			OpenFile ( const CSphString & sName, CSphString & sErrorBuffer );
-	void			SetFile ( int iFD, SphOffset_t * pSharedOffset );
+	bool			OpenFile ( const CSphString & sName, CSphString & sError );
+	void			SetFile ( CSphAutofile & tAuto, SphOffset_t * pSharedOffset, CSphString & sError );
 	void			CloseFile ( bool bTruncate = false );	///< note: calls Flush(), ie. IsError() might get true after this call
 	void			UnlinkFile (); /// some shit happened (outside) and the file is no more actual.
 
@@ -372,10 +372,10 @@ struct MemTracker_c : ISphNoncopyable
 #define DOCINFO_INDEX_FREQ 128 // FIXME? make this configurable
 
 
-inline uint64_t MVA_UPSIZE ( const DWORD * pMva )
+inline int64_t MVA_UPSIZE ( const DWORD * pMva )
 {
-	uint64_t uMva = (uint64_t)pMva[0] | ( ( (uint64_t)pMva[1] )<<32 );
-	return uMva;
+	int64_t iMva = (int64_t)( (uint64_t)pMva[0] | ( ( (uint64_t)pMva[1] )<<32 ) );
+	return iMva;
 }
 
 
@@ -412,10 +412,10 @@ private:
 	CSphVector<float>			m_dFloatMax;
 	CSphVector<float>			m_dFloatIndexMin;
 	CSphVector<float>			m_dFloatIndexMax;
-	CSphVector<uint64_t>		m_dMvaMin;
-	CSphVector<uint64_t>		m_dMvaMax;
-	CSphVector<uint64_t>		m_dMvaIndexMin;
-	CSphVector<uint64_t>		m_dMvaIndexMax;
+	CSphVector<int64_t>			m_dMvaMin;
+	CSphVector<int64_t>			m_dMvaMax;
+	CSphVector<int64_t>			m_dMvaIndexMin;
+	CSphVector<int64_t>			m_dMvaIndexMax;
 	DWORD						m_uStride;		// size of attribute's chunk (in DWORDs)
 	DWORD						m_uElements;	// counts total number of collected min/max pairs
 	int							m_iLoop;		// loop inside one set
@@ -467,6 +467,15 @@ typedef AttrIndexBuilder_t<> AttrIndexBuilder_c;
 #define LLONG_MAX (((unsigned long long)(-1))>>1)
 #endif
 
+#ifndef LLONG_MIN
+#define LLONG_MIN (-LLONG_MAX-1)
+#endif
+
+#ifndef ULLONG_MAX
+#define ULLONG_MAX	(LLONG_MAX * 2ULL + 1)
+#endif
+
+
 template < typename DOCID >
 void AttrIndexBuilder_t<DOCID>::ResetLocal()
 {
@@ -483,7 +492,7 @@ void AttrIndexBuilder_t<DOCID>::ResetLocal()
 	ARRAY_FOREACH ( i, m_dMvaMin )
 	{
 		m_dMvaMin[i] = LLONG_MAX;
-		m_dMvaMax[i] = 0;
+		m_dMvaMax[i] = ( i>=m_iMva64 ? LLONG_MIN : 0 );
 	}
 	m_uStart = m_uLast = 0;
 	m_iLoop = 0;
@@ -588,7 +597,7 @@ AttrIndexBuilder_t<DOCID>::AttrIndexBuilder_t ( const CSphSchema & tSchema )
 	for ( int i=0; i<tSchema.GetAttrsCount(); i++ )
 	{
 		const CSphColumnInfo & tCol = tSchema.GetAttr(i);
-		if ( tCol.m_eAttrType==SPH_ATTR_UINT64SET )
+		if ( tCol.m_eAttrType==SPH_ATTR_INT64SET )
 			m_dMvaAttrs.Add ( tCol.m_tLocator );
 	}
 
@@ -627,7 +636,7 @@ void AttrIndexBuilder_t<DOCID>::Prepare ( DWORD * pOutBuffer, DWORD * pOutMax )
 	ARRAY_FOREACH ( i, m_dMvaIndexMin )
 	{
 		m_dMvaIndexMin[i] = LLONG_MAX;
-		m_dMvaIndexMax[i] = 0;
+		m_dMvaIndexMax[i] = ( i>=m_iMva64 ? LLONG_MIN : 0 );
 	}
 	ResetLocal();
 }
@@ -668,9 +677,9 @@ void AttrIndexBuilder_t<DOCID>::CollectRowMVA ( int iAttr, DWORD uCount, const D
 		assert ( ( uCount%2 )==0 );
 		for ( ; uCount>0; uCount-=2, pMva+=2 )
 		{
-			uint64_t uVal = MVA_UPSIZE ( pMva );
-			m_dMvaMin[iAttr] = Min ( m_dMvaMin[iAttr], uVal );
-			m_dMvaMax[iAttr] = Max ( m_dMvaMax[iAttr], uVal );
+			int64_t iVal = MVA_UPSIZE ( pMva );
+			m_dMvaMin[iAttr] = Min ( m_dMvaMin[iAttr], iVal );
+			m_dMvaMax[iAttr] = Max ( m_dMvaMax[iAttr], iVal );
 		}
 	} else
 	{
@@ -1036,7 +1045,7 @@ inline const char * sphTypeName ( ESphAttr eType )
 		case SPH_ATTR_STRING:		return "string";
 		case SPH_ATTR_WORDCOUNT:	return "wordcount";
 		case SPH_ATTR_UINT32SET:	return "mva";
-		case SPH_ATTR_UINT64SET:	return "mva64";
+		case SPH_ATTR_INT64SET:		return "mva64";
 		default:					return "unknown";
 	}
 }
@@ -1055,7 +1064,7 @@ inline const char * sphTypeDirective ( ESphAttr eType )
 		case SPH_ATTR_STRING:		return "sql_attr_string";
 		case SPH_ATTR_WORDCOUNT:	return "sql_attr_wordcount";
 		case SPH_ATTR_UINT32SET:	return "sql_attr_multi";
-		case SPH_ATTR_UINT64SET:	return "sql_attr_multi bigint";
+		case SPH_ATTR_INT64SET:		return "sql_attr_multi bigint";
 		default:					return "???";
 	}
 }
@@ -1232,6 +1241,9 @@ void			sphMergeStats ( CSphQueryResultMeta & tDstResult, const SmallStringHash_T
 bool			sphCheckQueryHeight ( const struct XQNode_t * pRoot, CSphString & sError );
 void			sphTransformExtendedQuery ( XQNode_t ** ppNode );
 
+void			sphSetUnlinkOld ( bool bUnlink );
+void			sphUnlinkIndex ( const char * sName, bool bForce );
+
 void			WriteSchema ( CSphWriter & fdInfo, const CSphSchema & tSchema );
 void			ReadSchema ( CSphReader & rdInfo, CSphSchema & m_tSchema, DWORD uVersion, bool bDynamic );
 void			SaveIndexSettings ( CSphWriter & tWriter, const CSphIndexSettings & m_tSettings );
@@ -1409,8 +1421,12 @@ BYTE sphDoclistHintPack ( SphOffset_t iDocs, SphOffset_t iLen );
 /// startup mva updates arena
 const char *		sphArenaInit ( int iMaxBytes );
 
+#if USE_WINDOWS
+void localtime_r ( const time_t * clock, struct tm * res );
+#endif
+
 #endif // _sphinxint_
 
 //
-// $Id: sphinxint.h 3117 2012-02-22 20:30:36Z tomat $
+// $Id: sphinxint.h 3351 2012-08-19 13:48:59Z klirichek $
 //
