@@ -1,5 +1,5 @@
 //
-// $Id: sphinxquery.cpp 3114 2012-02-21 14:52:21Z klirichek $
+// $Id: sphinxquery.cpp 3429 2012-10-04 09:20:27Z kevg $
 //
 
 //
@@ -30,7 +30,7 @@ class XQParser_t
 {
 public:
 					XQParser_t ();
-					~XQParser_t () {}
+					~XQParser_t ();
 
 public:
 	bool			Parse ( XQQuery_t & tQuery, const char * sQuery, const ISphTokenizer * pTokenizer, const CSphSchema * pSchema, CSphDict * pDict, int iStopwordStep );
@@ -39,7 +39,7 @@ public:
 	void			Warning ( const char * sTemplate, ... ) __attribute__ ( ( format ( printf, 2, 3 ) ) );
 
 	bool			AddField ( CSphSmallBitvec & dFields, const char * szField, int iLen );
-	bool			ParseFields ( CSphSmallBitvec & uFields, int & iMaxFieldPos );
+	bool			ParseFields ( CSphSmallBitvec & uFields, int & iMaxFieldPos, bool & bIgnore );
 	int				ParseZone ( const char * pZone );
 
 	bool			IsSpecial ( char c );
@@ -53,14 +53,28 @@ public:
 	void			Cleanup ();
 	XQNode_t *		SweepNulls ( XQNode_t * pNode );
 	bool			FixupNots ( XQNode_t * pNode );
+	void			DeleteNodesWOFields ( XQNode_t * pNode );
 
 	inline void SetFieldSpec ( const CSphSmallBitvec& uMask, int iMaxPos )
 	{
-		m_dStateSpec.SetFieldSpec ( uMask, iMaxPos );
+		FixRefSpec();
+		m_dStateSpec.Last()->SetFieldSpec ( uMask, iMaxPos );
 	}
 	inline void SetZoneVec ( int iZoneVec )
 	{
-		m_dStateSpec.SetZoneSpec ( m_dZoneVecs[iZoneVec] );
+		FixRefSpec();
+		m_dStateSpec.Last()->SetZoneSpec ( m_dZoneVecs[iZoneVec] );
+	}
+
+	inline void FixRefSpec ()
+	{
+		bool bRef = ( m_dStateSpec.GetLength()>1 && ( m_dStateSpec[m_dStateSpec.GetLength()-1]==m_dStateSpec[m_dStateSpec.GetLength()-2] ) );
+		if ( !bRef )
+			return;
+
+		XQLimitSpec_t * pSpec = m_dStateSpec.Pop();
+		m_dSpecPool.Add ( new XQLimitSpec_t ( *pSpec ) );
+		m_dStateSpec.Add ( m_dSpecPool.Last() );
 	}
 
 public:
@@ -102,7 +116,8 @@ public:
 	CSphVector<CSphString>	m_dIntTokens;
 
 	CSphVector < CSphVector<int> >	m_dZoneVecs;
-	XQLimitSpec_t			m_dStateSpec;
+	CSphVector<XQLimitSpec_t *>		m_dStateSpec;
+	CSphVector<XQLimitSpec_t *>		m_dSpecPool;
 };
 
 //////////////////////////////////////////////////////////////////////////
@@ -262,6 +277,14 @@ XQParser_t::XQParser_t ()
 	, m_bQuoted ( false )
 	, m_bEmptyStopword ( false )
 {
+	m_dSpecPool.Add ( new XQLimitSpec_t() );
+	m_dStateSpec.Add ( m_dSpecPool.Last() );
+}
+
+XQParser_t::~XQParser_t ()
+{
+	ARRAY_FOREACH ( i, m_dSpecPool )
+		SafeDelete ( m_dSpecPool[i] );
 }
 
 
@@ -276,6 +299,7 @@ void XQParser_t::Cleanup ()
 		SafeDelete ( m_dSpawned[i] );
 	}
 	m_dSpawned.Reset ();
+	m_dStateSpec.Reset();
 }
 
 
@@ -350,10 +374,11 @@ bool XQParser_t::AddField ( CSphSmallBitvec & dFields, const char * szField, int
 
 
 /// parse fields block
-bool XQParser_t::ParseFields ( CSphSmallBitvec & dFields, int & iMaxFieldPos )
+bool XQParser_t::ParseFields ( CSphSmallBitvec & dFields, int & iMaxFieldPos, bool & bIgnore )
 {
 	dFields.Unset();
 	iMaxFieldPos = 0;
+	bIgnore = false;
 
 	const char * pPtr = m_pTokenizer->GetBufferPtr ();
 	const char * pLastPtr = m_pTokenizer->GetBufferEnd ();
@@ -387,6 +412,7 @@ bool XQParser_t::ParseFields ( CSphSmallBitvec & dFields, int & iMaxFieldPos )
 	// handle invalid chars
 	if ( !sphIsAlpha(*pPtr) )
 	{
+		bIgnore = true;
 		m_pTokenizer->SetBufferPtr ( pPtr ); // ignore and re-parse (FIXME! maybe warn?)
 		return true;
 	}
@@ -405,7 +431,7 @@ bool XQParser_t::ParseFields ( CSphSmallBitvec & dFields, int & iMaxFieldPos )
 			return false;
 
 		m_pTokenizer->SetBufferPtr ( pPtr );
-		if ( bNegate && ( !dFields.TestAll() ) )
+		if ( bNegate )
 			dFields.Negate();
 
 	} else
@@ -447,7 +473,7 @@ bool XQParser_t::ParseFields ( CSphSmallBitvec & dFields, int & iMaxFieldPos )
 					return false;
 
 				m_pTokenizer->SetBufferPtr ( ++pPtr );
-				if ( bNegate && ( !dFields.TestAll() ) )
+				if ( bNegate )
 					dFields.Negate();
 
 				bOK = true;
@@ -598,8 +624,9 @@ int XQParser_t::GetToken ( YYSTYPE * lvalp )
 		while ( p<sEnd && isdigit ( *(BYTE*)p ) ) p++;
 
 		static const int NUMBER_BUF_LEN = 10; // max strlen of int32
-
-		if ( p>sToken && p-sToken<NUMBER_BUF_LEN && ( *p=='\0' || isspace ( *(BYTE*)p ) || IsSpecial(*p) ) )
+		if ( p>sToken && p-sToken<NUMBER_BUF_LEN
+			&& !( *p=='-' && !( p-sToken==1 && sphIsModifier ( p[-1] ) ) ) // !bDashInside copied over from arbitration
+			&& ( *p=='\0' || sphIsSpace(*p) || IsSpecial(*p) ) )
 		{
 			if ( m_pTokenizer->GetToken() && m_pTokenizer->TokenIsBlended() ) // number with blended should be tokenized as usual
 			{
@@ -717,10 +744,15 @@ int XQParser_t::GetToken ( YYSTYPE * lvalp )
 			// some specials are especially special
 			if ( sToken[0]=='@' )
 			{
+				bool bIgnore;
+
 				// parse fields operator
-				if ( !ParseFields ( m_tPendingToken.tFieldLimit.dMask, m_tPendingToken.tFieldLimit.iMaxPos ) )
+				if ( !ParseFields ( m_tPendingToken.tFieldLimit.dMask, m_tPendingToken.tFieldLimit.iMaxPos, bIgnore ) )
 					return -1;
 
+				if ( bIgnore )
+					continue;
+
 				if ( m_pSchema->m_dFields.GetLength()!=SPH_MAX_FIELDS )
 					m_tPendingToken.tFieldLimit.dMask.LimitBits ( m_pSchema->m_dFields.GetLength() );
 
@@ -746,6 +778,17 @@ int XQParser_t::GetToken ( YYSTYPE * lvalp )
 					m_bQuoted = !m_bQuoted;
 				m_iPendingType = sToken[0]=='!' ? '-' : sToken[0];
 				m_pTokenizer->m_bPhrase = m_bQuoted;
+
+				if ( sToken[0]=='(' )
+				{
+					// safe way of performing m_dStateSpec.Add ( m_dStateSpec.Last() )
+					m_dStateSpec.Add ();
+					m_dStateSpec[m_dStateSpec.GetLength()-1] = m_dStateSpec[m_dStateSpec.GetLength()-2];
+				} else if ( sToken[0]==')' && m_dStateSpec.GetLength()>1 )
+				{
+					m_dStateSpec.Pop();
+				}
+
 				break;
 			}
 		}
@@ -769,7 +812,7 @@ int XQParser_t::GetToken ( YYSTYPE * lvalp )
 		// information about stars is lost after this point, so was have to save it now
 		DWORD uStarPosition = STAR_NONE;
 		uStarPosition |= *m_pTokenizer->GetTokenEnd()=='*' ? STAR_BACK : 0;
-		uStarPosition |= ( m_pTokenizer->GetTokenStart()!=m_pTokenizer->GetBufferPtr() ) &&
+		uStarPosition |= ( m_pTokenizer->GetTokenStart()>(const char *)m_sQuery ) &&
 			m_pTokenizer->GetTokenStart()[-1]=='*' ? STAR_FRONT : 0;
 
 		m_tPendingToken.pNode = AddKeyword ( sToken, uStarPosition );
@@ -812,7 +855,7 @@ XQNode_t * XQParser_t::AddKeyword ( const char * sKeyword, DWORD uStarPosition )
 	XQKeyword_t tAW ( sKeyword, m_iAtomPos );
 	tAW.m_uStarPosition = uStarPosition;
 
-	XQNode_t * pNode = new XQNode_t ( m_dStateSpec );
+	XQNode_t * pNode = new XQNode_t ( *m_dStateSpec.Last() );
 	pNode->m_dWords.Add ( tAW );
 
 	m_dSpawned.Add ( pNode );
@@ -843,7 +886,7 @@ XQNode_t * XQParser_t::AddOp ( XQOperator_e eOp, XQNode_t * pLeft, XQNode_t * pR
 
 	if ( eOp==SPH_QUERY_NOT )
 	{
-		XQNode_t * pNode = new XQNode_t ( m_dStateSpec );
+		XQNode_t * pNode = new XQNode_t ( *m_dStateSpec.Last() );
 		pNode->SetOp ( SPH_QUERY_NOT, pLeft );
 		m_dSpawned.Add ( pNode );
 		return pNode;
@@ -860,12 +903,6 @@ XQNode_t * XQParser_t::AddOp ( XQOperator_e eOp, XQNode_t * pLeft, XQNode_t * pR
 	// eg. '@title hello' vs 'world'
 	pRight->CopySpecs ( pLeft );
 
-	XQNode_t * pDonor = pRight;
-	if ( pRight->m_dSpec.m_bInvisible )
-		pDonor = pLeft;
-
-	m_dStateSpec = pDonor->m_dSpec;
-
 	// build a new node
 	XQNode_t * pResult = NULL;
 	if ( pLeft->m_dChildren.GetLength() && pLeft->GetOp()==eOp && pLeft->m_iOpArg==iOpArg )
@@ -881,7 +918,7 @@ XQNode_t * XQParser_t::AddOp ( XQOperator_e eOp, XQNode_t * pLeft, XQNode_t * pR
 	{
 		// however, it's right (!) spec which is chosen for the resulting node,
 		// eg. '@title hello' + 'world @body program'
-		XQNode_t * pNode = new XQNode_t ( pDonor->m_dSpec );
+		XQNode_t * pNode = new XQNode_t ( pRight->m_dSpec );
 		pNode->SetOp ( eOp, pLeft, pRight );
 		pNode->m_iOpArg = iOpArg;
 		m_dSpawned.Add ( pNode );
@@ -1012,17 +1049,20 @@ bool XQParser_t::FixupNots ( XQNode_t * pNode )
 }
 
 
-static void DeleteNodesWOFields ( XQNode_t * pNode )
+void XQParser_t::DeleteNodesWOFields ( XQNode_t * pNode )
 {
 	if ( !pNode )
 		return;
 
 	for ( int i = 0; i < pNode->m_dChildren.GetLength (); )
 	{
-		if ( pNode->m_dChildren[i]->m_dSpec.m_dFieldMask.TestAll() )
+		if ( pNode->m_dChildren[i]->m_dSpec.m_dFieldMask.TestAll ( false ) )
 		{
+			XQNode_t * pChild = pNode->m_dChildren[i];
+			assert ( pChild->m_dChildren.GetLength()==0 );
+			m_dSpawned.RemoveValue ( pChild );
+
 			// this should be a leaf node
-			assert ( pNode->m_dChildren[i]->m_dChildren.GetLength()==0 );
 			SafeDelete ( pNode->m_dChildren[i] );
 			pNode->m_dChildren.RemoveFast ( i );
 
@@ -1142,7 +1182,7 @@ bool XQParser_t::Parse ( XQQuery_t & tParsed, const char * sQuery, const ISphTok
 
 	// all ok; might want to create a dummy node to indicate that
 	m_dSpawned.Reset();
-	tParsed.m_pRoot = m_pRoot ? m_pRoot : new XQNode_t ( m_dStateSpec );
+	tParsed.m_pRoot = m_pRoot ? m_pRoot : new XQNode_t ( *m_dStateSpec.Last() );
 	return true;
 }
 
@@ -1178,7 +1218,7 @@ static void xqDump ( XQNode_t * pNode, const CSphSchema & tSch, int iIndent )
 	} else
 	{
 		xqIndent ( iIndent );
-		printf ( "MATCH(%d,%d):", pNode->m_uFieldMask, pNode->m_iOpArg );
+		printf ( "MATCH(%d,%d):", pNode->m_dSpec.m_dFieldMask.GetMask32(), pNode->m_iOpArg );
 
 		ARRAY_FOREACH ( i, pNode->m_dWords )
 		{
@@ -1215,6 +1255,10 @@ bool sphParseExtendedQuery ( XQQuery_t & tParsed, const char * sQuery, const ISp
 	}
 #endif
 
+	// moved here from ranker creation
+	// as at that point term expansion could produce many terms from expanded term and this condition got failed
+	tParsed.m_bSingleWord = ( tParsed.m_pRoot && tParsed.m_pRoot->m_dChildren.GetLength()==0 && tParsed.m_pRoot->m_dWords.GetLength()==1 );
+
 	return bRes;
 }
 
@@ -1759,5 +1803,5 @@ int sphMarkCommonSubtrees ( int iXQ, const XQQuery_t * pXQ )
 }
 
 //
-// $Id: sphinxquery.cpp 3114 2012-02-21 14:52:21Z klirichek $
+// $Id: sphinxquery.cpp 3429 2012-10-04 09:20:27Z kevg $
 //
