@@ -1,5 +1,5 @@
 //
-// $Id: sphinxquery.h 3114 2012-02-21 14:52:21Z klirichek $
+// $Id: sphinxquery.h 3302 2012-07-25 12:21:45Z tomat $
 //
 
 //
@@ -80,7 +80,6 @@ enum XQOperator_e
 struct XQLimitSpec_t
 {
 	bool					m_bFieldSpec;	///< whether field spec was already explicitly set
-	bool					m_bInvisible;	///< totally ignore this set
 	CSphSmallBitvec			m_dFieldMask;	///< fields mask (spec part)
 	int						m_iFieldMaxPos;	///< max position within field (spec part)
 	CSphVector<int>			m_dZones;		///< zone indexes in per-query zones list
@@ -93,18 +92,12 @@ public:
 
 	inline void Reset ()
 	{
-		m_bInvisible = false;
 		m_bFieldSpec = false;
 		m_iFieldMaxPos = 0;
 		m_dFieldMask.Set();
 		m_dZones.Reset();
 	}
 
-	inline void Hide ()
-	{
-		m_bInvisible = true;
-	}
-
 	XQLimitSpec_t ( const XQLimitSpec_t& dLimit )
 	{
 		if ( this==&dLimit )
@@ -158,7 +151,7 @@ public:
 
 public:
 	/// ctor
-	explicit XQNode_t ( const XQLimitSpec_t& dSpec )
+	explicit XQNode_t ( const XQLimitSpec_t & dSpec )
 		: m_pParent ( NULL )
 		, m_eOp ( SPH_QUERY_AND )
 		, m_iOrder ( 0 )
@@ -268,11 +261,13 @@ struct XQQuery_t : public ISphNoncopyable
 
 	CSphVector<CSphString>	m_dZones;
 	XQNode_t *				m_pRoot;
+	bool					m_bSingleWord;
 
 	/// ctor
 	XQQuery_t ()
 	{
 		m_pRoot = NULL;
+		m_bSingleWord = false;
 	}
 
 	/// dtor
@@ -295,5 +290,5 @@ int		sphMarkCommonSubtrees ( int iXQ, const XQQuery_t * pXQ );
 #endif // _sphinxquery_
 
 //
-// $Id: sphinxquery.h 3114 2012-02-21 14:52:21Z klirichek $
+// $Id: sphinxquery.h 3302 2012-07-25 12:21:45Z tomat $
 //
