@@ -1165,7 +1165,7 @@ yyreduce:
 
   case 24:
 
-    { yyval.pNode = yyvsp[-1].pNode; if ( yyval.pNode ) yyval.pNode->m_dSpec.Hide(); pParser->m_dStateSpec.Reset(); ;}
+    { yyval.pNode = yyvsp[-1].pNode; ;}
     break;
 
   case 26:
